@@ -16,6 +16,7 @@
 #include "log.h"
 #include "tpm2.h"
 #include "tpm2_tool.h"
+#include "tpm2_util.h"
 
 /**
  * This is the magic for the file header. The header is organized
@@ -163,7 +164,7 @@ bool file_read_bytes_from_file(FILE *f, UINT8 *buf, UINT16 *size,
     return true;
 }
 
-bool files_load_bytes_from_path(const char *path, UINT8 *buf, UINT16 *size) {
+TEST_WEAK bool files_load_bytes_from_path(const char *path, UINT8 *buf, UINT16 *size) {
 
     if (!buf || !size || !path) {
         return false;
@@ -199,7 +200,7 @@ bool files_save_bytes_to_file(const char *path, UINT8 *buf, UINT16 size) {
 
     bool result = files_write_bytes(fp, buf, size);
     if (!result) {
-        LOG_ERR("Could not write data to file \"%s\"", path);
+        LOG_ERR("Could not write data to file \"%s\"", path ? path : "<stdout>");
     }
 
     if (fp != stdout) {
@@ -494,7 +495,7 @@ bool files_does_file_exist(const char *path) {
     return false;
 }
 
-bool files_get_file_size_path(const char *path, unsigned long *file_size) {
+TEST_WEAK bool files_get_file_size_path(const char *path, unsigned long *file_size) {
 
     bool result = false;
 
@@ -564,6 +565,15 @@ bool files_read_bytes(FILE *out, UINT8 bytes[], size_t len) {
     return (readx(out, bytes, len) == len);
 }
 
+bool files_read_bytes_chunk(FILE *out, UINT8 bytes[], size_t len, size_t *read_len) {
+
+    BAIL_ON_NULL("FILE", out);
+    BAIL_ON_NULL("bytes", bytes);
+    size_t chunk_len = readx(out, bytes, len);
+    *read_len += chunk_len;
+    return (chunk_len == len);
+}
+
 bool files_write_bytes(FILE *out, uint8_t bytes[], size_t len) {
 
     BAIL_ON_NULL("FILE", out);
