@@ -12,6 +12,8 @@
 #include "tpm2_alg_util.h"
 #include "tpm2_attr_util.h"
 #include "tpm2_errata.h"
+#include "tpm2_hash.h"
+#include "tpm2_policy.h"
 
 typedef struct alg_pair alg_pair;
 struct alg_pair {
@@ -42,7 +44,7 @@ static void tpm2_alg_util_for_each_alg(alg_iter iterator, void *userdata) {
 
     static const alg_pair algs[] = {
 
-        // Assymetric
+        // Asymmetric
         { .name = "rsa", .id = TPM2_ALG_RSA, .flags = tpm2_alg_util_flags_asymmetric|tpm2_alg_util_flags_base },
         { .name = "ecc", .id = TPM2_ALG_ECC, .flags = tpm2_alg_util_flags_asymmetric|tpm2_alg_util_flags_base },
 
@@ -50,6 +52,7 @@ static void tpm2_alg_util_for_each_alg(alg_iter iterator, void *userdata) {
         { .name = "tdes", .id = TPM2_ALG_TDES, .flags = tpm2_alg_util_flags_symmetric },
         { .name = "aes", .id = TPM2_ALG_AES, .flags = tpm2_alg_util_flags_symmetric },
         { .name = "camellia", .id = TPM2_ALG_CAMELLIA, .flags = tpm2_alg_util_flags_symmetric },
+        { .name = "sm4", .id = TPM2_ALG_SM4, .flags = tpm2_alg_util_flags_symmetric },
 
         // Hash
         { .name = "sha1", .id = TPM2_ALG_SHA1, .flags = tpm2_alg_util_flags_hash },
@@ -75,17 +78,13 @@ static void tpm2_alg_util_for_each_alg(alg_iter iterator, void *userdata) {
         { .name = "ecdsa", .id = TPM2_ALG_ECDSA, .flags = tpm2_alg_util_flags_sig },
         { .name = "ecdaa", .id = TPM2_ALG_ECDAA, .flags = tpm2_alg_util_flags_sig },
         { .name = "ecschnorr", .id = TPM2_ALG_ECSCHNORR, .flags = tpm2_alg_util_flags_sig },
+        { .name = "sm2", .id = TPM2_ALG_SM2, .flags = tpm2_alg_util_flags_sig },
 
-        // Assyemtric Encryption Scheme
+        // Asymmetric Encryption Scheme
         { .name = "oaep", .id = TPM2_ALG_OAEP, .flags = tpm2_alg_util_flags_enc_scheme | tpm2_alg_util_flags_rsa_scheme },
         { .name = "rsaes", .id = TPM2_ALG_RSAES, .flags = tpm2_alg_util_flags_enc_scheme | tpm2_alg_util_flags_rsa_scheme },
         { .name = "ecdh", .id = TPM2_ALG_ECDH, .flags = tpm2_alg_util_flags_enc_scheme },
 
-
-        // XXX are these sigs?
-        { .name = "sm2", .id = TPM2_ALG_SM2, .flags = tpm2_alg_util_flags_sig },
-        { .name = "sm4", .id = TPM2_ALG_SM4, .flags = tpm2_alg_util_flags_sig },
-
         // Key derivation functions
         { .name = "kdf1_sp800_56a", .id = TPM2_ALG_KDF1_SP800_56A, .flags = tpm2_alg_util_flags_kdf },
         { .name = "kdf2", .id = TPM2_ALG_KDF2, .flags = tpm2_alg_util_flags_kdf },
@@ -117,7 +116,7 @@ static void tpm2_alg_util_for_each_alg(alg_iter iterator, void *userdata) {
     }
 }
 
-static alg_parser_rc handle_sym_common(const char *ext, TPMT_SYM_DEF_OBJECT *s) {
+static alg_parser_rc handle_sym_common(const char *ext, TPMT_SYM_DEF_OBJECT *s, bool is_parent) {
 
     if (ext == NULL || ext[0] == '\0') {
         ext = "128";
@@ -136,7 +135,7 @@ static alg_parser_rc handle_sym_common(const char *ext, TPMT_SYM_DEF_OBJECT *s)
     ext += 3;
 
     if (*ext == '\0') {
-        ext = "null";
+        ext = is_parent ? "cfb" : "null";
     }
 
     s->mode.sym = tpm2_alg_util_strtoalg(ext,
@@ -200,6 +199,8 @@ static alg_parser_rc handle_scheme_sign(const char *scheme,
             do_scheme_halg(scheme, 4, TPM2_ALG_ECDH);
         } else if (!strncmp(scheme, "ecschnorr", 9)) {
             do_scheme_halg(scheme, 9, TPM2_ALG_ECSCHNORR);
+        } else if (!strncmp(scheme, "sm2", 3)) {
+            do_scheme_halg(scheme, 3, TPM2_ALG_SM2);
         } else if (!strncmp(scheme, "ecdaa", 5)) {
             do_scheme_halg(scheme, 5, TPM2_ALG_ECDAA);
             /*
@@ -317,14 +318,33 @@ static alg_parser_rc handle_ecc(const char *ext, TPM2B_PUBLIC *public) {
 
     public->publicArea.type = TPM2_ALG_ECC;
 
-    size_t len = ext ? strlen(ext) : 0;
-    if (len == 0 || ext[0] == '\0') {
-        ext = "256";
-    }
-
     TPMS_ECC_PARMS *e = &public->publicArea.parameters.eccDetail;
     e->kdf.scheme = TPM2_ALG_NULL;
+    e->curveID = TPM2_ECC_NONE;
 
+    /* handle default ecc curve (NIST_P256) */
+    if (ext == NULL || ext[0] == '\0') {
+        e->curveID = TPM2_ECC_NIST_P256;
+        return alg_parser_rc_continue;
+    }
+
+    if (ext[0] == '_') {
+        /* skip separator */
+        ext++;
+        if (!strcmp(ext, "sm2_p256") || !strcmp(ext, "sm2")) {
+            e->curveID = TPM2_ECC_SM2_P256;
+            return alg_parser_rc_continue;
+        } else if (strncmp(ext, "nist_p", 6)) {
+            return alg_parser_rc_error;
+        }
+        ext += 6;
+        if (ext[0] == '\0') {
+            return alg_parser_rc_error;
+        }
+        /* fall through to NIST curves */
+    }
+
+    /* parse NIST curves */
     if (!strncmp(ext, "192", 3)) {
         e->curveID = TPM2_ECC_NIST_P192;
         ext += 3;
@@ -340,8 +360,6 @@ static alg_parser_rc handle_ecc(const char *ext, TPM2B_PUBLIC *public) {
     } else if (!strncmp(ext, "521", 3)) {
         e->curveID = TPM2_ECC_NIST_P521;
         ext += 3;
-    } else {
-        e->curveID = TPM2_ECC_NIST_P256;
     }
 
     /* ecc extension should be consumed at this point */
@@ -358,7 +376,7 @@ static alg_parser_rc handle_aes(const char *ext, TPM2B_PUBLIC *public) {
     TPMT_SYM_DEF_OBJECT *s = &public->publicArea.parameters.symDetail.sym;
     s->algorithm = TPM2_ALG_AES;
 
-    return handle_sym_common(ext, s);
+    return handle_sym_common(ext, s, false);
 }
 
 static alg_parser_rc handle_camellia(const char *ext, TPM2B_PUBLIC *public) {
@@ -368,7 +386,17 @@ static alg_parser_rc handle_camellia(const char *ext, TPM2B_PUBLIC *public) {
     TPMT_SYM_DEF_OBJECT *s = &public->publicArea.parameters.symDetail.sym;
     s->algorithm = TPM2_ALG_CAMELLIA;
 
-    return handle_sym_common(ext, s);
+    return handle_sym_common(ext, s, false);
+}
+
+static alg_parser_rc handle_sm4(const char *ext, TPM2B_PUBLIC *public) {
+
+    public->publicArea.type = TPM2_ALG_SYMCIPHER;
+
+    TPMT_SYM_DEF_OBJECT *s = &public->publicArea.parameters.symDetail.sym;
+    s->algorithm = TPM2_ALG_SM4;
+
+    return handle_sym_common(ext, s, false);
 }
 
 static alg_parser_rc handle_xor(TPM2B_PUBLIC *public) {
@@ -408,6 +436,9 @@ static alg_parser_rc handle_object(const char *object, TPM2B_PUBLIC *public) {
     } else if (!strncmp(object, "camellia", 8)) {
         object += 8;
         return handle_camellia(object, public);
+    } else if (!strncmp(object, "sm4", 3)) {
+        object += (object[3] == '_') ? 4 : 3;
+        return handle_sm4(object, public);
     } else if (!strcmp(object, "hmac")) {
         return handle_hmac(public);
     } else if (!strcmp(object, "xor")) {
@@ -484,10 +515,14 @@ static alg_parser_rc handle_asym_detail(const char *detail,
 
         if (!strncmp(detail, "aes", 3)) {
             s->algorithm = TPM2_ALG_AES;
-            return handle_sym_common(detail + 3, s);
+            return handle_sym_common(detail + 3, s, is_restricted);
         } else if (!strncmp(detail, "camellia", 8)) {
             s->algorithm = TPM2_ALG_CAMELLIA;
-            return handle_sym_common(detail + 8, s);
+            return handle_sym_common(detail + 8, s, is_restricted);
+        } else if (!strncmp(detail, "sm4", 3)) {
+            s->algorithm = TPM2_ALG_SM4;
+            detail += (detail[3] == '_') ? 4 : 3;
+            return handle_sym_common(detail, s, is_restricted);
         } else if (!strcmp(detail, "null")) {
             s->algorithm = TPM2_ALG_NULL;
             return alg_parser_rc_done;
@@ -601,6 +636,10 @@ tool_rc tpm2_alg_util_handle_rsa_ext_alg(const char *alg_spec,
     #define RSA_KEYBITS_STRLEN 6
     char *ext_alg_str = calloc(1, strlen(alg_spec) + strlen("rsa") +
         RSA_KEYBITS_STRLEN);
+    if (ext_alg_str == NULL) {
+        LOG_ERR("oom");
+        return tool_rc_general_error;
+    }
 
     strcat(ext_alg_str, "rsa");
     switch(public->publicArea.parameters.rsaDetail.keyBits) {
@@ -669,6 +708,16 @@ const char *tpm2_alg_util_algtostr(TPM2_ALG_ID id, tpm2_alg_util_flags flags) {
     return userdata.name;
 }
 
+const char *tpm2_alg_util_numtoalgstr(const char *str, tpm2_alg_util_flags flags) {
+    TPM2_ALG_ID alg_id;
+
+    if (tpm2_util_string_to_uint16(str, &alg_id)) {
+        return tpm2_alg_util_algtostr(alg_id, flags);
+    } else {
+        return str;
+    }
+}
+
 tpm2_alg_util_flags tpm2_alg_util_algtoflags(TPM2_ALG_ID id) {
 
     alg_pair userdata = { .name = NULL, .id = id, .flags =
@@ -962,20 +1011,18 @@ tool_rc tpm2_alg_util_get_signature_scheme(ESYS_CONTEXT *ectx,
     return tool_rc_success;
 }
 
-tool_rc tpm2_alg_util_public_init(char *alg_details, char *name_halg, char *attrs,
-        char *auth_policy,  TPMA_OBJECT def_attrs, TPM2B_PUBLIC *public) {
+tool_rc tpm2_alg_util_public_init(const char *alg_details, const char *name_halg,
+        char *attrs, char *auth_policy,  TPMA_OBJECT def_attrs,
+        TPM2B_PUBLIC *public) {
 
     memset(public, 0, sizeof(*public));
 
     /* load a policy from a path if present */
     if (auth_policy) {
-    public->publicArea.authPolicy.size =
-                sizeof(public->publicArea.authPolicy.buffer);
-        bool res = files_load_bytes_from_path(auth_policy,
-            public->publicArea.authPolicy.buffer,
-                &public->publicArea.authPolicy.size);
-        if (!res) {
-            return tool_rc_general_error;
+        tool_rc rc = tpm2_policy_set_digest(auth_policy,
+                &public->publicArea.authPolicy);
+        if (rc != tool_rc_success) {
+            return rc;
         }
     }
 
@@ -1066,3 +1113,128 @@ bool tpm2_alg_util_is_aes_size_valid(UINT16 size_in_bytes) {
         return false;
     }
 }
+
+bool tpm2_alg_util_is_sm4_size_valid(UINT16 size_in_bytes) {
+
+    switch (size_in_bytes) {
+    case 16:
+        return true;
+    default:
+        LOG_ERR("Invalid SM4 key size, got %u bytes, expected 16",
+                size_in_bytes);
+        return false;
+    }
+}
+
+TPM2_ALG_ID tpm2_alg_util_get_name_alg(ESYS_CONTEXT *ectx, ESYS_TR handle) {
+
+    TPM2B_NAME *name = NULL;
+    TSS2_RC rc = Esys_TR_GetName(ectx, handle, &name);
+    if (rc != TSS2_RC_SUCCESS) {
+        return TPM2_ALG_ERROR;
+    }
+
+    if (name->size < 2) {
+        Esys_Free(name);
+        return TPM2_ALG_ERROR;
+    }
+
+    UINT16 *big_endian_alg = (UINT16 *)name->name;
+
+    TPM2_ALG_ID name_alg = tpm2_util_ntoh_16(*big_endian_alg);
+    Esys_Free(name);
+    return name_alg;
+}
+
+tool_rc tpm2_alg_util_sm2_compute_id_digest(ESYS_CONTEXT *ectx, ESYS_TR handle, const char *id, size_t idlen, TPM2B_DIGEST *result) {
+
+    TPMS_ALGORITHM_DETAIL_ECC *parameters = NULL;
+    TPM2B_PUBLIC *public = NULL;
+    TPM2B *id_input = NULL;
+    TPM2B_DIGEST *tmp_digest = NULL;
+
+    if (!id || !result) {
+        return tool_rc_general_error;
+    }
+
+    /* 2-byte id length in bits */
+    if (strlen(id) != idlen) {
+        LOG_ERR("invalid sm2 id!");
+        return tool_rc_general_error;
+    }
+
+    if (idlen > SM2_MAX_ID_LENGTH || idlen <= 0) {
+        LOG_ERR("invalid id length!");
+        return tool_rc_general_error;
+    }
+
+    tool_rc rc = tpm2_geteccparameters(ectx, TPM2_ECC_SM2_P256, &parameters,
+        NULL, TPM2_ALG_NULL);
+    if (rc != tool_rc_success) {
+        LOG_ERR("Could not get ecc parameters!");
+        goto out;
+    }
+
+    rc = tpm2_readpublic(ectx, handle, &public, NULL, NULL);
+    if (rc != tool_rc_success) {
+        LOG_ERR("Could not read public!");
+        goto out;
+    }
+
+    UINT16 key_nbytes = parameters->keySize/8;
+    if (public->publicArea.type != TPM2_ALG_ECC ||
+            public->publicArea.parameters.eccDetail.curveID != TPM2_ECC_SM2_P256 ||
+            public->publicArea.unique.ecc.x.size != key_nbytes ||
+            public->publicArea.unique.ecc.y.size != key_nbytes) {
+        LOG_ERR("invalid sm2 public key!");
+        rc = tool_rc_general_error;
+        goto out;
+    }
+
+    BYTE idbits[2];
+    idbits[0] = ((idlen * 8) >> 8) % 256;
+    idbits[1] = (idlen * 8) % 256;
+
+    UINT16 total_size = sizeof(idbits) + idlen + 6 * key_nbytes;
+
+    id_input = (TPM2B *) calloc(1, sizeof(TPM2B) + total_size);
+    if (id_input == NULL) {
+        LOG_ERR("Could not calloc memory!");
+        rc = tool_rc_general_error;
+        goto out;
+    }
+
+    id_input->size = total_size;
+
+    UINT16 pos = 0;
+    memcpy(id_input->buffer + pos, idbits, sizeof(idbits));
+    pos += sizeof(idbits);
+    memcpy(id_input->buffer + pos, id, idlen);
+    pos += idlen;
+    memcpy(id_input->buffer + pos, parameters->a.buffer, key_nbytes);
+    pos += key_nbytes;
+    memcpy(id_input->buffer + pos, parameters->b.buffer, key_nbytes);
+    pos += key_nbytes;
+    memcpy(id_input->buffer + pos, parameters->gX.buffer, key_nbytes);
+    pos += key_nbytes;
+    memcpy(id_input->buffer + pos, parameters->gY.buffer, key_nbytes);
+    pos += key_nbytes;
+    memcpy(id_input->buffer + pos, public->publicArea.unique.ecc.x.buffer, key_nbytes);
+    pos += key_nbytes;
+    memcpy(id_input->buffer + pos, public->publicArea.unique.ecc.y.buffer, key_nbytes);
+
+    rc = tpm2_hash_compute_data(ectx, TPM2_ALG_SM3_256, TPM2_RH_OWNER,
+                        id_input->buffer, id_input->size, &tmp_digest, NULL);
+    if (rc != tool_rc_success) {
+        goto out;
+    }
+
+    *result = *tmp_digest;
+
+out:
+    free(id_input);
+    Esys_Free(public);
+    Esys_Free(tmp_digest);
+    Esys_Free(parameters);
+    return rc;
+}
