@@ -45,6 +45,16 @@ TPM2_ALG_ID tpm2_alg_util_strtoalg(const char *name, tpm2_alg_util_flags flags);
  */
 const char *tpm2_alg_util_algtostr(TPM2_ALG_ID id, tpm2_alg_util_flags flags);
 
+/**
+ * If the input is a numerical string then convert it to a nice-name.
+ * Otherwise return the unmodified nice-name.
+ * @param str
+ *  The numerical string to convert.
+ * @return
+ *  The nice-name.
+ */
+const char *tpm2_alg_util_numtoalgstr(const char* str, tpm2_alg_util_flags flags);
+
 /**
  * XXX DOC AND TESTME
  * @param id
@@ -185,8 +195,9 @@ tool_rc tpm2_alg_util_handle_rsa_ext_alg(const char *alg_spec,
  * @param public
  * @return
  */
-tool_rc tpm2_alg_util_public_init(char *alg_details, char *name_halg, char *attrs,
-        char *auth_policy,  TPMA_OBJECT def_attrs, TPM2B_PUBLIC *public);
+tool_rc tpm2_alg_util_public_init(const char *alg_details, const char *name_halg,
+        char *attrs, char *auth_policy,  TPMA_OBJECT def_attrs,
+        TPM2B_PUBLIC *public);
 
 /**
  * Returns an ECC curve as a friendly name.
@@ -206,4 +217,53 @@ const char *tpm2_alg_util_ecc_to_str(TPM2_ECC_CURVE curve_id);
  */
 bool tpm2_alg_util_is_aes_size_valid(UINT16 size_in_bytes);
 
+/**
+ * Determines if a size is a valid SM4 key size.
+ * @param size_in_bytes
+ *  The size of a potential SM4 key in bytes.
+ * @return
+ *  true if valid, false otherwise.
+ */
+bool tpm2_alg_util_is_sm4_size_valid(UINT16 size_in_bytes);
+
+/**
+ * Given an ESYS_TR handle to an object, retrieves the name algorithm
+ * without making a readpublic call.
+ *
+ * @param ectx
+ *  The ESAPI context.
+ * @param handle
+ *  The handle of the object to query.
+ * @return
+ *  The TPM2_ALG_ID name algorithm identifier or TPM2_ALG_ERROR for an error.
+ */
+TPM2_ALG_ID tpm2_alg_util_get_name_alg(ESYS_CONTEXT *ectx, ESYS_TR handle);
+
+#define SM2_MAX_ID_BITS             65535
+#define SM2_MAX_ID_LENGTH           (SM2_MAX_ID_BITS/8)
+#define SM2_DEFAULT_ID_GMT09            "1234567812345678"
+#define SM2_DEFAULT_ID              SM2_DEFAULT_ID_GMT09
+#define SM2_DEFAULT_ID_LENGTH           (sizeof(SM2_DEFAULT_ID) - 1)
+
+/**
+ * Given an ESYS_TR handle to an object, retrieves ecc sm2 parameters
+ * with making a geteccparameters call, retrieves the public key with
+ * making a readpublic call, and compute id digest.
+ *
+ * @param ectx
+ *  The ESAPI context.
+ * @param handle
+ *  The handle of the object to query.
+ * @param id
+ *  sm2 id.
+ * @param idlen
+ *  sm2 id length.
+ * @param result
+ *  sm2 id digest output.
+ * @return
+ *  tool_rc indicating status.
+ */
+tool_rc tpm2_alg_util_sm2_compute_id_digest(ESYS_CONTEXT *ectx, ESYS_TR handle,
+        const char *id, size_t idlen, TPM2B_DIGEST *result);
+
 #endif /* LIB_TPM2_ALG_UTIL_H_ */
