@@ -23,6 +23,26 @@
 #include <efivar/efivar.h>
 #endif
 
+/* Valid variable unicode names and their length */
+#define NAME_DB "db"
+#define NAME_DB_LEN 2
+#define NAME_DBX "dbx"
+#define NAME_DBX_LEN 3
+#define NAME_BOOTORDER "BootOrder"
+#define NAME_BOOTORDER_LEN 9
+#define NAME_KEK "KEK"
+#define NAME_KEK_LEN 3
+#define NAME_MOKLISTTRUSTED "MokListTrusted"
+#define NAME_MOKLISTTRUSTED_LEN 14
+#define NAME_PK "PK"
+#define NAME_PK_LEN 2
+#define NAME_SBATLEVEL "SbatLevel"
+#define NAME_SBATLEVEL_LEN 9
+#define NAME_SECUREBOOT "SecureBoot"
+#define NAME_SHIM "Shim"
+#define NAME_SHIM_LEN 4
+#define NAME_SECUREBOOT_LEN 10
+
 static void guid_unparse_lower(EFI_GUID guid, char guid_buf[37]) {
 
     snprintf(guid_buf, 37, "%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x",
@@ -268,6 +288,123 @@ char *yaml_devicepath(BYTE* dp, UINT64 dp_len) {
     return text_path; 
 }
 #endif
+/*
+ * The yaml_ipl description is received as raw bytes, but the
+ * data will represent a printable string. Unfortunately we
+ * are not told its encoding, and this can vary. For example,
+ * grub will use UTF8, while sd-boot will UTF16LE.
+ *
+ * We need to emit YAML with some rules:
+ *
+ *  - No leading ' ' without quoting it
+ *  - Escape non-printable ascii chars
+ *  - Double quotes if using escape sequences
+ *  - Valid UTF8 string
+ *
+ * This method will ignore the question of original data
+ * encoding and apply a few simple rules to make the data
+ * mostly YAML compliant. Where it falls down is not
+ * guaranteeing valid UTF8, if the input was not already
+ * valid UTF8. In practice this limitation shouldn't be
+ * a problem given expected measured data.
+ *
+ * Note: one consequence of this approach is that most
+ * UTF16LE data will be rendered with lots of \0 bytes
+ * escaped.
+ *
+ * For ease of output reading, the data is also split on newlines
+ */
+char **yaml_split_escape_string(UINT8 const *description, size_t size)
+{
+    char **lines = NULL, **tmp;
+    size_t nlines = 0;
+    size_t i, j, k;
+    size_t len;
+    UINT8 *nl;
+
+    i = 0;
+    do {
+        nl = memchr(description + i, '\n', size - i);
+        len = nl ? (size_t)(nl - (description + i)) : size - i;
+
+        tmp = realloc(lines, sizeof(char *) * (nlines + 2));
+        if (!tmp) {
+            LOG_ERR("failed to allocate memory for description lines: %s\n",
+                    strerror(errno));
+            goto error;
+        }
+        lines = tmp;
+        lines[nlines + 1] = NULL;
+        k = 0;
+
+        /* Worst case: every byte needs escaping, plus start/end quotes, plus nul */
+        lines[nlines] = calloc(1, (len * 2) + 2 + 1);
+        if (!lines[nlines]) {
+            LOG_ERR("failed to allocate memory for escaped string: %s\n",
+                    strerror(errno));
+            goto error;
+        }
+
+        lines[nlines][k++] = '"';
+        for (j = i; j < (i + len); j++) {
+            char escape = '\0';
+
+            switch (description[j]) {
+            case '\0':
+              escape = '0';
+              break;
+            case '\a':
+              escape = 'a';
+              break;
+            case '\b':
+              escape = 'b';
+              break;
+            case '\t':
+              escape = 't';
+              break;
+            case '\v':
+              escape = 'v';
+              break;
+            case '\f':
+              escape = 'f';
+              break;
+            case '\r':
+              escape = 'r';
+              break;
+            case '\e':
+              escape = 'e';
+              break;
+            case '\'':
+              escape = '\'';
+              break;
+            case '\\':
+              escape = '\\';
+              break;
+            }
+
+            if (escape == '\0') {
+                lines[nlines][k++] = description[j];
+            } else {
+                lines[nlines][k++] = '\\';
+                lines[nlines][k++] = escape;
+            }
+        }
+        lines[nlines][k++] = '"';
+
+        nlines++;
+        i += len + 1;
+    } while (i < size);
+
+    return lines;
+
+ error:
+    for (i = 0; lines != NULL && lines[i] != NULL; i++) {
+      free(lines[i]);
+    }
+    free(lines);
+    return NULL;
+}
+
 /*
  * TCG PC Client FPF section 9.2.6
  * The tpm2_eventlog module validates the event structure but nothing within
@@ -321,10 +458,10 @@ static bool yaml_uefi_var(UEFI_VARIABLE_DATA *data, size_t size, UINT32 type,
          * respectively.
          */
         if (type == EV_EFI_VARIABLE_DRIVER_CONFIG) {
-            if ((strlen(ret) == 2 && strncmp(ret, "PK", 2) == 0) ||
-                (strlen(ret) == 3 && strncmp(ret, "KEK", 3) == 0) ||
-                (strlen(ret) == 2 && strncmp(ret, "db", 2) == 0) ||
-                (strlen(ret) == 3 && strncmp(ret, "dbx", 3) == 0)) {
+            if ((strlen(ret) == NAME_PK_LEN && strncmp(ret, NAME_PK, NAME_PK_LEN) == 0) ||
+                (strlen(ret) == NAME_KEK_LEN && strncmp(ret, NAME_KEK, NAME_KEK_LEN) == 0) ||
+                (strlen(ret) == NAME_DB_LEN && strncmp(ret, NAME_DB, NAME_DB_LEN) == 0) ||
+                (strlen(ret) == NAME_DBX_LEN && strncmp(ret, NAME_DBX, NAME_DBX_LEN) == 0)) {
 
                 free(ret);
                 tpm2_tool_output("    VariableData:\n");
@@ -374,13 +511,13 @@ static bool yaml_uefi_var(UEFI_VARIABLE_DATA *data, size_t size, UINT32 type,
                     for (i = 0; i < signatures; i++) {
                         EFI_SIGNATURE_DATA *s = (EFI_SIGNATURE_DATA *)signature;
                         char *sdata = calloc (1,
-                            BYTES_TO_HEX_STRING_SIZE(slist->SignatureSize-16));
+                            BYTES_TO_HEX_STRING_SIZE(slist->SignatureSize - sizeof(EFI_GUID)));
                         if (sdata == NULL) {
                             LOG_ERR("Failled to allocate data: %s\n", strerror(errno));
                             return false;
                         }
-                        bytes_to_str(s->SignatureData, slist->SignatureSize-16,
-                            sdata, BYTES_TO_HEX_STRING_SIZE(slist->SignatureSize-16));
+                        bytes_to_str(s->SignatureData, slist->SignatureSize - sizeof(EFI_GUID),
+                            sdata, BYTES_TO_HEX_STRING_SIZE(slist->SignatureSize - sizeof(EFI_GUID)));
                         guid_unparse_lower(s->SignatureOwner, uuidstr);
                         tpm2_tool_output("      - SignatureOwner: %s\n"
                                          "        SignatureData: %s\n",
@@ -397,7 +534,7 @@ static bool yaml_uefi_var(UEFI_VARIABLE_DATA *data, size_t size, UINT32 type,
                     variable_data += slist->SignatureListSize;
                 }
                 return true;
-            } else if ((strlen(ret) == 10 && strncmp(ret, "SecureBoot", 10) == 0)) {
+            } else if ((strlen(ret) == NAME_SECUREBOOT_LEN && strncmp(ret, NAME_SECUREBOOT, NAME_SECUREBOOT_LEN) == 0)) {
                 free(ret);
                 tpm2_tool_output("    VariableData:\n"
                                  "      Enabled: ");
@@ -418,29 +555,76 @@ static bool yaml_uefi_var(UEFI_VARIABLE_DATA *data, size_t size, UINT32 type,
                 }
                 return true;
             }
-            /* Other variables will be printed as a hex string */
         } else if (type == EV_EFI_VARIABLE_AUTHORITY) {
-            free(ret);
-            tpm2_tool_output("    VariableData:\n");
-            
-            EFI_SIGNATURE_DATA *s= (EFI_SIGNATURE_DATA *)&data->UnicodeName[
-                data->UnicodeNameLength];
-            char *sdata = calloc (1,
-                BYTES_TO_HEX_STRING_SIZE(data->VariableDataLength - 16));
-            if (sdata == NULL) {
-                LOG_ERR("Failled to allocate data: %s\n", strerror(errno));
-                return false;
+            /* The MokListTrusted is boolean option, not a EFI_SIGNATURE_DATA*/
+            if ((strlen(ret) == NAME_MOKLISTTRUSTED_LEN && strncmp(ret, NAME_MOKLISTTRUSTED, NAME_MOKLISTTRUSTED_LEN) == 0)) {
+                free(ret);
+                tpm2_tool_output("    VariableData:\n"
+                                 "      Enabled: ");
+                if (data->VariableDataLength == 0) {
+                    tpm2_tool_output("'No'\n");
+                } else if (data->VariableDataLength > 1) {
+                    LOG_ERR("MokListTrusted value length %" PRIu64 " is unexpectedly > 1\n",
+                            data->VariableDataLength);
+                    return false;
+                } else {
+                    uint8_t *variable_data = (uint8_t *)&data->UnicodeName[
+                        data->UnicodeNameLength];
+                    if (*variable_data == 0) {
+                        tpm2_tool_output("'No'\n");
+                    } else {
+                        tpm2_tool_output("'Yes'\n");
+                    }
+                }
+                return true;
+            } else if ((strlen(ret) == NAME_DB_LEN && strncmp(ret, NAME_DB, NAME_DB_LEN) == 0) ||
+                       (strlen(ret) == NAME_SHIM_LEN && strncmp(ret, NAME_SHIM, NAME_SHIM_LEN) == 0)) {
+                /* db and Shim will be parsed as EFI_SIGNATURE_DATA */
+                free(ret);
+                tpm2_tool_output("    VariableData:\n");
+                EFI_SIGNATURE_DATA *s= (EFI_SIGNATURE_DATA *)&data->UnicodeName[
+                    data->UnicodeNameLength];
+                if (data->VariableDataLength < sizeof(EFI_SIGNATURE_DATA)) {
+                    LOG_ERR("VariableDataLength is too short for EFI_SIGNATURE_DATA");
+                    return false;
+                }
+                char *sdata = calloc (1,
+                    BYTES_TO_HEX_STRING_SIZE(data->VariableDataLength - sizeof(EFI_GUID)));
+                if (sdata == NULL) {
+                    LOG_ERR("Failled to allocate data: %s\n", strerror(errno));
+                    return false;
+                }
+                bytes_to_str(s->SignatureData, data->VariableDataLength - sizeof(EFI_GUID),
+                    sdata, BYTES_TO_HEX_STRING_SIZE(data->VariableDataLength - sizeof(EFI_GUID)));
+                guid_unparse_lower(s->SignatureOwner, uuidstr);
+                tpm2_tool_output("    - SignatureOwner: %s\n"
+                                 "      SignatureData: %s\n",
+                                uuidstr, sdata);
+                free(sdata);
+                return true;
+            } else if (strlen(ret) == NAME_SBATLEVEL_LEN && strncmp(ret, NAME_SBATLEVEL, NAME_SBATLEVEL_LEN) == 0)  {
+                free(ret);
+                tpm2_tool_output("    VariableData:\n"
+                                 "      String: |-\n");
+
+                UINT8 *description = (UINT8 *)&data->UnicodeName[
+                    data->UnicodeNameLength];
+                char **lines = NULL;
+                lines = yaml_split_escape_string(description, data->VariableDataLength);
+                if (!lines) {
+                    return false;
+                }
+
+                size_t i;
+                for (i = 0; lines[i] != NULL; i++) {
+                    tpm2_tool_output("        %s\n", lines[i]);
+                    free(lines[i]);
+                }
+                free(lines);
+                return true;
             }
-            bytes_to_str(s->SignatureData, data->VariableDataLength - 16,
-                sdata, BYTES_TO_HEX_STRING_SIZE(data->VariableDataLength - 16));
-            guid_unparse_lower(s->SignatureOwner, uuidstr);
-            tpm2_tool_output("    - SignatureOwner: %s\n"
-                             "      SignatureData: %s\n",
-                             uuidstr, sdata);
-            free(sdata);
-            return true;
         } else if (type == EV_EFI_VARIABLE_BOOT || type == EV_EFI_VARIABLE_BOOT2) {
-            if ((strlen(ret) == 9 && strncmp(ret, "BootOrder", 9) == 0)) {
+            if ((strlen(ret) == NAME_BOOTORDER_LEN && strncmp(ret, NAME_BOOTORDER, NAME_BOOTORDER_LEN) == 0)) {
                 free(ret);
                 tpm2_tool_output("    VariableData:\n");
                 
@@ -571,6 +755,7 @@ bool yaml_uefi_action(UINT8 const *action, size_t size) {
 
     return true;
 }
+
 /*
  * TCG PC Client PFP section 9.4.1
  * This event type is extensively used by the Shim and Grub on a wide varities
@@ -578,22 +763,22 @@ bool yaml_uefi_action(UINT8 const *action, size_t size) {
  * the loading of grub, kernel, and initrd images.
  */
 bool yaml_ipl(UINT8 const *description, size_t size) {
-
+    char **lines = NULL;
+    size_t i;
     tpm2_tool_output("  Event:\n"
                      "    String: |-\n");
 
-    /* We need to handle when description contains multiple lines. */
-    size_t i, j;
-    for (i = 0; i < size; i++) {
-        for (j = i; j < size; j++) {
-            if (description[j] == '\n' || description[j] == '\0') {
-                break;
-            }
-        }
-        tpm2_tool_output("      %.*s\n", (int)(j - i), description+i);
-        i = j;
+    lines = yaml_split_escape_string(description, size);
+    if (!lines) {
+        return false;
     }
 
+    for (i = 0; lines[i] != NULL; i++) {
+        tpm2_tool_output("      %s\n", lines[i]);
+        free(lines[i]);
+    }
+    free(lines);
+
     return true;
 }
 /* TCG PC Client PFP section 9.2.3 */
