@@ -6,6 +6,7 @@
 
 #include "files.h"
 #include "log.h"
+#include "tool_rc.h"
 #include "tpm2.h"
 #include "tpm2_alg_util.h"
 #include "tpm2_openssl.h"
@@ -174,7 +175,8 @@ tool_rc tpm2_policy_build_pcr(ESYS_CONTEXT *ectx, tpm2_session *policy_session,
 tool_rc tpm2_policy_build_policyauthorize(ESYS_CONTEXT *ectx,
         tpm2_session *policy_session, const char *policy_digest_path,
         const char *qualifying_data,
-        const char *verifying_pubkey_name_path, const char *ticket_path) {
+        const char *verifying_pubkey_name_path, const char *ticket_path,
+        TPM2B_DIGEST *cp_hash, TPMI_ALG_HASH parameter_hash_algorithm) {
 
     bool result = true;
     TPM2B_DIGEST approved_policy = { .size = 0 };
@@ -236,7 +238,7 @@ tool_rc tpm2_policy_build_policyauthorize(ESYS_CONTEXT *ectx,
     ESYS_TR sess_handle = tpm2_session_get_handle(policy_session);
     return tpm2_policy_authorize(ectx, sess_handle, ESYS_TR_NONE, ESYS_TR_NONE,
             ESYS_TR_NONE, &approved_policy, &policy_qualifier, &key_sign,
-            &check_ticket);
+            &check_ticket, cp_hash, parameter_hash_algorithm);
 }
 
 tool_rc tpm2_policy_build_policyor(ESYS_CONTEXT *ectx,
@@ -248,12 +250,13 @@ tool_rc tpm2_policy_build_policyor(ESYS_CONTEXT *ectx,
 }
 
 tool_rc tpm2_policy_build_policypassword(ESYS_CONTEXT *ectx,
-        tpm2_session *session) {
+        tpm2_session *session, TPM2B_DIGEST *cp_hash,
+        TPMI_ALG_HASH parameter_hash_algorithm) {
 
     ESYS_TR policy_session_handle = tpm2_session_get_handle(session);
 
     return tpm2_policy_password(ectx, policy_session_handle, ESYS_TR_NONE,
-            ESYS_TR_NONE, ESYS_TR_NONE);
+            ESYS_TR_NONE, ESYS_TR_NONE, cp_hash, parameter_hash_algorithm);
 }
 
 tool_rc tpm2_policy_build_policynamehash(ESYS_CONTEXT *ectx,
@@ -281,12 +284,13 @@ tool_rc tpm2_policy_build_policycphash(ESYS_CONTEXT *ectx,
 }
 
 tool_rc tpm2_policy_build_policyauthvalue(ESYS_CONTEXT *ectx,
-        tpm2_session *session) {
+        tpm2_session *session, TPM2B_DIGEST *cp_hash,
+        TPMI_ALG_HASH parameter_hash_algorithm) {
 
     ESYS_TR policy_session_handle = tpm2_session_get_handle(session);
 
     return tpm2_policy_authvalue(ectx, policy_session_handle, ESYS_TR_NONE,
-            ESYS_TR_NONE, ESYS_TR_NONE);
+            ESYS_TR_NONE, ESYS_TR_NONE, cp_hash, parameter_hash_algorithm);
 }
 
 tool_rc tpm2_policy_build_policysecret(ESYS_CONTEXT *ectx,
@@ -501,30 +505,33 @@ tool_rc tpm2_policy_get_digest(ESYS_CONTEXT *ectx, tpm2_session *session,
 }
 
 tool_rc tpm2_policy_build_policycommandcode(ESYS_CONTEXT *ectx,
-        tpm2_session *session, uint32_t command_code) {
+        tpm2_session *session, uint32_t command_code, TPM2B_DIGEST *cp_hash,
+        TPMI_ALG_HASH parameter_hash_algorithm) {
 
     ESYS_TR handle = tpm2_session_get_handle(session);
 
     return tpm2_policy_command_code(ectx, handle, ESYS_TR_NONE, ESYS_TR_NONE,
-            ESYS_TR_NONE, command_code);
+            ESYS_TR_NONE, command_code, cp_hash, parameter_hash_algorithm);
 }
 
-tool_rc tpm2_policy_build_policynvwritten(ESYS_CONTEXT *ectx,
-        tpm2_session *session, TPMI_YES_NO written_set) {
+tool_rc  tpm2_policy_build_policynvwritten(ESYS_CONTEXT *ectx,
+        tpm2_session *session, TPMI_YES_NO written_set, TPM2B_DIGEST *cp_hash,
+        TPMI_ALG_HASH parameter_hash_algorithm) {
 
     ESYS_TR handle = tpm2_session_get_handle(session);
 
     return tpm2_policy_nv_written(ectx, handle, ESYS_TR_NONE, ESYS_TR_NONE,
-            ESYS_TR_NONE, written_set);
+            ESYS_TR_NONE, written_set, cp_hash, parameter_hash_algorithm);
 }
 
 tool_rc tpm2_policy_build_policylocality(ESYS_CONTEXT *ectx,
-        tpm2_session *session, TPMA_LOCALITY locality) {
+        tpm2_session *session, TPMA_LOCALITY locality, TPM2B_DIGEST *cp_hash,
+        TPMI_ALG_HASH parameter_hash_algorithm) {
 
     ESYS_TR handle = tpm2_session_get_handle(session);
 
     return tpm2_policy_locality(ectx, handle, ESYS_TR_NONE, ESYS_TR_NONE,
-            ESYS_TR_NONE, locality);
+            ESYS_TR_NONE, locality, cp_hash, parameter_hash_algorithm);
 }
 
 tool_rc tpm2_policy_build_policyduplicationselect(ESYS_CONTEXT *ectx,
@@ -685,3 +692,15 @@ error:
     free(policy_digest);
     return rc;
 }
+
+tool_rc tpm2_policy_set_digest(const char *auth_policy, TPM2B_DIGEST *out_policy) {
+
+    if (!auth_policy) {
+        memset(out_policy, 0, sizeof(*out_policy));
+        return tool_rc_success;
+    }
+
+    out_policy->size = sizeof(out_policy->buffer);
+    bool res = tpm2_util_bin_from_hex_or_file(auth_policy, &out_policy->size, out_policy->buffer);
+    return res ? tool_rc_success : tool_rc_general_error;
+}
