@@ -51,7 +51,8 @@ tool_rc tpm2_policy_build_pcr(ESYS_CONTEXT *context,
 tool_rc tpm2_policy_build_policyauthorize(ESYS_CONTEXT *ectx,
         tpm2_session *policy_session, const char *policy_digest_path,
         const char *policy_qualifier,
-        const char *verifying_pubkey_name_path, const char *ticket_path);
+        const char *verifying_pubkey_name_path, const char *ticket_path,
+        TPM2B_DIGEST *cp_hash, TPMI_ALG_HASH parameter_hash_algorithm);
 
 /**
  * Compounds policies in an OR fashion
@@ -166,7 +167,8 @@ tool_rc tpm2_policy_get_digest(ESYS_CONTEXT *context, tpm2_session *session,
  *  tool_rc indicating status.
  */
 tool_rc tpm2_policy_build_policypassword(ESYS_CONTEXT *ectx,
-        tpm2_session *session);
+        tpm2_session *session, TPM2B_DIGEST *cp_hash,
+        TPMI_ALG_HASH parameter_hash_algorithm);
 
 /**
  * Enables a policy that requires the object's authvalue be provided.
@@ -179,7 +181,8 @@ tool_rc tpm2_policy_build_policypassword(ESYS_CONTEXT *ectx,
  *  tool_rc indicating status.
  */
 tool_rc tpm2_policy_build_policyauthvalue(ESYS_CONTEXT *ectx,
-        tpm2_session *session);
+        tpm2_session *session, TPM2B_DIGEST *cp_hash,
+        TPMI_ALG_HASH parameter_hash_algorithm);
 
 /**
  * Enables a policy authorization by virtue of verifying a signature on optional
@@ -249,7 +252,8 @@ bool tpm2_policy_parse_policy_list(char *str, TPML_DIGEST *policy_list);
  *  A tool_rc indicating status.
  */
 tool_rc tpm2_policy_build_policycommandcode(ESYS_CONTEXT *ectx,
-        tpm2_session *session, uint32_t command_code);
+        tpm2_session *session, uint32_t command_code, TPM2B_DIGEST *cp_hash,
+        TPMI_ALG_HASH parameter_hash_algorithm);
 
 /**
  * Policy to restrict authorization to written state of the NV Index
@@ -264,7 +268,8 @@ tool_rc tpm2_policy_build_policycommandcode(ESYS_CONTEXT *ectx,
  *  A tool_rc indicating status.
  */
 tool_rc tpm2_policy_build_policynvwritten(ESYS_CONTEXT *ectx,
-        tpm2_session *session, TPMI_YES_NO written_set);
+        tpm2_session *session, TPMI_YES_NO written_set, TPM2B_DIGEST *cp_hash,
+        TPMI_ALG_HASH parameter_hash_algorithm);
 
 /**
  * Policy to restrict tpm object authorization to specific locality
@@ -279,7 +284,8 @@ tool_rc tpm2_policy_build_policynvwritten(ESYS_CONTEXT *ectx,
  *   A tool_rc indicating status.
  */
 tool_rc tpm2_policy_build_policylocality(ESYS_CONTEXT *ectx,
-        tpm2_session *session, TPMA_LOCALITY locality);
+        tpm2_session *session, TPMA_LOCALITY locality, TPM2B_DIGEST *cp_hash,
+        TPMI_ALG_HASH parameter_hash_algorithm);
 
 /**
  * Policy to restrict tpm object authorization to specific duplication target
@@ -320,4 +326,16 @@ tool_rc tpm2_policy_build_policyduplicationselect(ESYS_CONTEXT *ectx,
 tool_rc tpm2_policy_tool_finish(ESYS_CONTEXT *ectx, tpm2_session *session,
         const char *save_path);
 
+/** Sets a TPM2B_DIGEST from a file if present or a hex string.
+ *
+ * @param auth_policy
+ *  Either a file path or a hex string. A NULL pointer causes out_policy to be memset
+ *  to 0.
+ * @param out_policy
+ *  The set policy
+ * @return
+ *  tool_rc_success on success, or any tool_rc failure on failure.
+ */
+tool_rc tpm2_policy_set_digest(const char *auth_policy, TPM2B_DIGEST *out_policy);
+
 #endif /* TPM2_POLICY_H_ */
