@@ -5,7 +5,8 @@ source helpers.sh
 cleanup() {
     rm -f primary.ctx new_parent.prv new_parent.pub new_parent.ctx policy.dat \
     session.dat key.prv key.pub key.ctx duppriv.bin dupseed.dat key2.prv \
-    key2.pub key2.ctx sym_key_in.bin
+    key2.pub key2.ctx sym_key_in.bin cleartext.txt secret.bin decrypted.txt \
+    primary.pub rsa-priv.pem rsa.pub rsa.priv rsa.dpriv rsa.seed rsa-pub.pem rsa.sig
 
     if [ "$1" != "no-shut-down" ]; then
           shut_down
@@ -100,8 +101,107 @@ tpm2 duplicate -Q -C new_parent.ctx -c key2.ctx -G aes -o sym_key_out.bin \
 -p "session:session.dat" -r dupprv.bin -s dupseed.dat
 end_duplication_session
 
+## External RSA key, wrapped for the primary key
+tpm2 readpublic -c primary.ctx -o primary.pub
+openssl genrsa -out rsa-priv.pem
+openssl rsa -in rsa-priv.pem -pubout > rsa-pub.pem
+tpm2 duplicate \
+	--tcti none \
+	-U primary.pub \
+	-G rsa \
+	-k rsa-priv.pem \
+	-u rsa.pub \
+	-r rsa.dpriv \
+	-s rsa.seed
+tpm2 import \
+	-C primary.ctx \
+	-G rsa \
+	-i rsa.dpriv \
+	-s rsa.seed \
+	-u rsa.pub \
+	-r rsa.priv
+
+# validate that TPM signatures with this imported key are acceptable to OpenSSL
+tpm2 load \
+	-C primary.ctx \
+	-c rsa.ctx \
+	-u rsa.pub \
+	-r rsa.priv
+echo foo | tpm2 sign \
+	-c rsa.ctx \
+	-o rsa.sig \
+	-f plain
+echo foo | openssl dgst \
+	-sha256 \
+	-verify rsa-pub.pem \
+	-signature rsa.sig
+
+## External RSA key, with a password authorization policy
+echo magicwords > cleartext.txt
+openssl pkeyutl \
+	-encrypt \
+	-pubin \
+	-inkey rsa-pub.pem \
+	-in cleartext.txt \
+	-out secret.bin
+
+tpm2 startauthsession -S session.dat
+tpm2 policypassword -S session.dat -L policy.dat
+
+tpm2 duplicate \
+	--tcti none \
+	-U primary.pub \
+	-G rsa \
+	-k rsa-priv.pem \
+	-u rsa.pub \
+	-r rsa.dpriv \
+	-s rsa.seed \
+	-L policy.dat \
+	-p secretpassword
+
+tpm2 import \
+	-C primary.ctx \
+	-G rsa \
+	-i rsa.dpriv \
+	-s rsa.seed \
+	-u rsa.pub \
+	-r rsa.priv
+
+# validate that TPM can decrypt messages with this imported key
+
+tpm2 load \
+	-C primary.ctx \
+	-c rsa.ctx \
+	-u rsa.pub \
+	-r rsa.priv \
+
+tpm2 startauthsession -S session.dat --policy-session
+tpm2 policypassword -S session.dat -L policy.dat
+
+cat secret.bin | tpm2 rsadecrypt \
+	-c rsa.ctx \
+	-p session:session.dat+secretpassword \
+	> decrypted.txt
+cmp cleartext.txt decrypted.txt
+
 trap - ERR
 
+## Attempt to decrypt without the password or policy
+
+cat secret.bin | tpm2 rsadecrypt \
+	-c rsa.ctx \
+	-p session:session.dat
+if [ $? -eq 0 ]; then
+  echo "Expected \"tpm2 rsadecrypt\" to fail without password."
+  exit 1
+fi
+cat secret.bin | tpm2 rsadecrypt \
+	-c rsa.ctx
+if [ $? -eq 0 ]; then
+  echo "Expected \"tpm2 rsadecrypt\" to fail without policy."
+  exit 1
+fi
+
 ## Null parent - should fail (TPM_RC_HIERARCHY)
 start_duplication_session
 tpm2 duplicate -Q -C null -c key2.ctx -G aes -i sym_key_in.bin \
