@@ -57,6 +57,7 @@ static tpm2_verifysig_ctx ctx = {
 static bool verify(void) {
 
     bool result = false;
+    EVP_PKEY_CTX *pkey_ctx = NULL;
 
     /* read the public key */
     EVP_PKEY *pkey = NULL;
@@ -65,7 +66,19 @@ static bool verify(void) {
         return false;
     }
 
-    EVP_PKEY_CTX *pkey_ctx = EVP_PKEY_CTX_new(pkey, NULL);
+#if OPENSSL_VERSION_NUMBER >= 0x10101003L
+#if OPENSSL_VERSION_MAJOR < 3
+    if (ctx.halg == TPM2_ALG_SM3_256) {
+        ret = EVP_PKEY_set_alias_type(pkey, EVP_PKEY_SM2);
+        if (!ret) {
+            LOG_ERR("EVP_PKEY_set_alias_type failed: %s", ERR_error_string(ERR_get_error(), NULL));
+            goto err;
+        }
+    }
+#endif
+#endif
+
+    pkey_ctx = EVP_PKEY_CTX_new(pkey, NULL);
     if (!pkey_ctx) {
         LOG_ERR("EVP_PKEY_CTX_new failed: %s", ERR_error_string(ERR_get_error(), NULL));
         goto err;
@@ -74,7 +87,7 @@ static bool verify(void) {
     /* get the digest alg */
     /* TODO SPlit loading on plain vs tss format to detect the hash alg */
     /* If its a plain sig we need -g */
-    const EVP_MD *md = tpm2_openssl_halg_from_tpmhalg(ctx.halg);
+    const EVP_MD *md = tpm2_openssl_md_from_tpmhalg(ctx.halg);
     // TODO error handling
 
     int rc = EVP_PKEY_verify_init(pkey_ctx);
@@ -269,11 +282,11 @@ static bool parse_selection_data_from_file(FILE *pcr_input,
 
     if (le64toh(pcrs->count) > ARRAY_LEN(pcrs->pcr_values)) {
         LOG_ERR("Malformed PCR file, pcr count cannot be greater than %zu, got: %" PRIu64 " ",
-                ARRAY_LEN(pcrs->pcr_values), le64toh(pcrs->count));
+                ARRAY_LEN(pcrs->pcr_values), le64toh((UINT64)pcrs->count));
         return false;
     }
 
-    UINT32 j;
+    size_t j;
     for (j = 0; j < le64toh(pcrs->count); j++) {
         if (fread(&pcrs->pcr_values[j], sizeof(TPML_DIGEST), 1, pcr_input)
                 != 1) {
@@ -376,7 +389,7 @@ static tool_rc init(void) {
     TPM2B_ATTEST *msg = NULL;
     TPML_PCR_SELECTION pcr_select;
     tpm2_pcrs *pcrs;
-    tpm2_pcrs temp_pcrs;
+    tpm2_pcrs temp_pcrs = {};
     tool_rc return_value = tool_rc_general_error;
 
     msg = message_from_file(ctx.msg_file_path);
@@ -463,7 +476,7 @@ static tool_rc init(void) {
         if (pcr_select.count > TPM2_NUM_PCR_BANKS)
             goto err;
 
-        tpm2_eventlog_context eventlog_ctx = {};
+        tpm2_eventlog_context eventlog_ctx = { 0 };
         bool rc = eventlog_from_file(&eventlog_ctx, ctx.eventlog_path);
         if (!rc) {
             LOG_ERR("Failed to process eventlog");
