@@ -241,7 +241,14 @@ static bool retrieve_web_endorsement_certificate(char *b64h) {
         }
     }
 
-    snprintf(weblink, len, "%s%s%s", ctx.ek_server_addr, "/", b64h);
+    bool is_slash_append_required =
+        strncmp((ctx.ek_server_addr + strlen(ctx.ek_server_addr) - 1), "/", 1);
+    if (is_slash_append_required) {
+        snprintf(weblink, len, "%s%s%s", ctx.ek_server_addr, "/", b64h);
+    } else {
+        snprintf(weblink, len, "%s%s", ctx.ek_server_addr, b64h);
+    }
+
     rc = curl_easy_setopt(curl, CURLOPT_URL, weblink);
     if (rc != CURLE_OK) {
         LOG_ERR("curl_easy_setopt for CURLOPT_URL failed: %s",
@@ -432,11 +439,11 @@ static tool_rc nv_read(ESYS_CONTEXT *ectx, TPMI_RH_NV_INDEX nv_index) {
 
          tpm2_util_nv_read(ectx, nv_index, 0, 0, &object, &ctx.rsa_cert_buffer,
             &ctx.rsa_cert_buffer_size, &cp_hash, &rp_hash, TPM2_ALG_SHA256, 0,
-            ESYS_TR_NONE, ESYS_TR_NONE) :
+            ESYS_TR_NONE, ESYS_TR_NONE, NULL) :
 
          tpm2_util_nv_read(ectx, nv_index, 0, 0, &object, &ctx.ecc_cert_buffer,
             &ctx.ecc_cert_buffer_size, &cp_hash, &rp_hash, TPM2_ALG_SHA256, 0,
-            ESYS_TR_NONE, ESYS_TR_NONE);
+            ESYS_TR_NONE, ESYS_TR_NONE, NULL);
 
 nv_read_out:
     tmp_rc = tpm2_session_close(&object.session);
@@ -519,12 +526,6 @@ static tool_rc get_ek_certificates(ESYS_CONTEXT *ectx) {
     /*
      * Following is everything applicable to ctx.is_cert_on_nv = false.
      */
-
-    rc = print_intel_ek_certificate_warning();
-    if (rc != tool_rc_success) {
-        return rc;
-    }
-
     if (!ctx.ek_path) {
         LOG_ERR("Must specify the EK public key path");
         return tool_rc_option_error;
